@@ -1,6 +1,10 @@
 use std::fs;
+use std::sync::Arc;
 
 use crate::node::Node;
+use crate::progress::Operation;
+use crate::progress::PAtomicInfo;
+use crate::progress::ORDERING;
 use crate::utils::is_filtered_out_due_to_invert_regex;
 use crate::utils::is_filtered_out_due_to_regex;
 use rayon::iter::ParallelBridge;
@@ -8,16 +12,12 @@ use rayon::prelude::ParallelIterator;
 use regex::Regex;
 use std::path::PathBuf;
 
-use std::sync::atomic;
-use std::sync::atomic::AtomicBool;
-
 use std::collections::HashSet;
 
 use crate::node::build_node;
 use std::fs::DirEntry;
 
 use crate::platform::get_metadata;
-
 pub struct WalkData<'a> {
     pub ignore_directories: HashSet<PathBuf>,
     pub filter_regex: &'a [Regex],
@@ -26,22 +26,25 @@ pub struct WalkData<'a> {
     pub use_apparent_size: bool,
     pub by_filecount: bool,
     pub ignore_hidden: bool,
+    pub follow_links: bool,
+    pub progress_data: Arc<PAtomicInfo>,
 }
 
-pub fn walk_it(dirs: HashSet<PathBuf>, walk_data: WalkData) -> (Vec<Node>, bool) {
-    let permissions_flag = AtomicBool::new(false);
-
+pub fn walk_it(dirs: HashSet<PathBuf>, walk_data: WalkData) -> Vec<Node> {
+    let mut inodes = HashSet::new();
     let top_level_nodes: Vec<_> = dirs
         .into_iter()
         .filter_map(|d| {
-            clean_inodes(
-                walk(d, &permissions_flag, &walk_data, 0)?,
-                &mut HashSet::new(),
-                walk_data.use_apparent_size,
-            )
+            let prog_data = &walk_data.progress_data;
+            prog_data.clear_state(&d);
+            let node = walk(d, &walk_data, 0)?;
+
+            prog_data.state.store(Operation::PREPARING, ORDERING);
+
+            clean_inodes(node, &mut inodes, walk_data.use_apparent_size)
         })
         .collect();
-    (top_level_nodes, permissions_flag.into_inner())
+    top_level_nodes
 }
 
 // Remove files which have the same inode, we don't want to double count them.
@@ -121,12 +124,8 @@ fn ignore_file(entry: &DirEntry, walk_data: &WalkData) -> bool {
     (is_dot_file && walk_data.ignore_hidden) || is_ignored_path
 }
 
-fn walk(
-    dir: PathBuf,
-    permissions_flag: &AtomicBool,
-    walk_data: &WalkData,
-    depth: usize,
-) -> Option<Node> {
+fn walk(dir: PathBuf, walk_data: &WalkData, depth: usize) -> Option<Node> {
+    let prog_data = &walk_data.progress_data;
     let mut children = vec![];
 
     if let Ok(entries) = fs::read_dir(&dir) {
@@ -139,38 +138,42 @@ fn walk(
                     // rayon doesn't parallelize as well giving a 3X performance drop
                     // hence we unravel the recursion a bit
 
-                    // return walk(entry.path(), permissions_flag, ignore_directories, allowed_filesystems, use_apparent_size, by_filecount, ignore_hidden);
+                    // return walk(entry.path(), walk_data, depth)
 
                     if !ignore_file(entry, walk_data) {
                         if let Ok(data) = entry.file_type() {
-                            return if data.is_dir() && !data.is_symlink() {
-                                walk(entry.path(), permissions_flag, walk_data, depth + 1)
-                            } else {
-                                build_node(
-                                    entry.path(),
-                                    vec![],
-                                    walk_data.filter_regex,
-                                    walk_data.invert_filter_regex,
-                                    walk_data.use_apparent_size,
-                                    data.is_symlink(),
-                                    data.is_file(),
-                                    walk_data.by_filecount,
-                                    depth,
-                                )
-                            };
+                            if data.is_dir() || (walk_data.follow_links && data.is_symlink()) {
+                                return walk(entry.path(), walk_data, depth + 1);
+                            }
+
+                            let node = build_node(
+                                entry.path(),
+                                vec![],
+                                walk_data.filter_regex,
+                                walk_data.invert_filter_regex,
+                                walk_data.use_apparent_size,
+                                data.is_symlink(),
+                                data.is_file(),
+                                walk_data.by_filecount,
+                                depth,
+                            );
+
+                            prog_data.num_files.fetch_add(1, ORDERING);
+                            if let Some(ref file) = node {
+                                prog_data.total_file_size.fetch_add(file.size, ORDERING);
+                            }
+
+                            return node;
                         }
                     }
                 } else {
-                    permissions_flag.store(true, atomic::Ordering::Relaxed);
+                    prog_data.no_permissions.store(true, ORDERING)
                 }
                 None
             })
             .collect();
-    } else {
-        // Handle edge case where dust is called with a file instead of a directory
-        if !dir.exists() {
-            permissions_flag.store(true, atomic::Ordering::Relaxed);
-        }
+    } else if !dir.is_file() {
+        walk_data.progress_data.no_permissions.store(true, ORDERING)
     }
     build_node(
         dir,
