@@ -12,11 +12,13 @@ mod utils;
 
 use crate::cli::build_cli;
 use dir_walker::WalkData;
+use display::InitialDisplayData;
 use filter::AggregateData;
 use progress::PIndicator;
 use progress::ORDERING;
 use std::collections::HashSet;
 use std::io::BufRead;
+use std::panic;
 use std::process;
 use sysinfo::{System, SystemExt};
 
@@ -129,7 +131,7 @@ fn main() {
         .value_of_t("width")
         .unwrap_or_else(|_| get_width_of_terminal());
 
-    let depth = options.value_of_t("depth").unwrap_or(usize::MAX);
+    let depth = config.get_depth(&options);
 
     // If depth is set, then we set the default number_of_lines to be max
     // instead of screen height
@@ -189,7 +191,10 @@ fn main() {
         progress_data: indicator.data.clone(),
     };
 
-    let _rayon = init_rayon();
+    let result = panic::catch_unwind(|| init_rayon);
+    if result.is_err() {
+        eprintln!("Problem initializing rayon, try: export RAYON_NUM_THREADS=1")
+    }
 
     let top_level_nodes = walk_it(simplified_dirs, walk_data);
 
@@ -217,17 +222,20 @@ fn main() {
     }
 
     if let Some(root_node) = tree {
+        let idd = InitialDisplayData {
+            short_paths: !config.get_full_paths(&options),
+            is_reversed: !config.get_reverse(&options),
+            colors_on: !no_colors,
+            by_filecount,
+            iso,
+            is_screen_reader: config.get_screen_reader(&options),
+        };
         draw_it(
-            config.get_full_paths(&options),
-            !config.get_reverse(&options),
-            no_colors,
+            idd,
             config.get_no_bars(&options),
             terminal_width,
-            by_filecount,
             &root_node,
-            iso,
             config.get_skip_total(&options),
-            config.get_screen_reader(&options),
         )
     }
 }
@@ -244,6 +252,6 @@ fn init_rayon() -> Result<(), ThreadPoolBuildError> {
             .stack_size(large_stack)
             .build_global()
     } else {
-        Ok(())
+        rayon::ThreadPoolBuilder::new().build_global()
     }
 }
